#ifndef _PYGAME2_DOCDRAW_H_
#define _PYGAME2_DOCDRAW_H_

#define DOC_DRAW "Extension module containing simple 2D drawing methods for the pygame2\nSDL modules. Several methods utilize code from the SDL_gfx library, and\nwere optimised and changed in several ways."
#define DOC_DRAW_AALINE "aaline (surface, color, x1, y1, x2, y2[, blend]) -> pygame2.Rect\naaline (surface, color, point1, point2[, blend]) -> pygame2.Rect\n\nDraws a fine anti-aliased line.\n\nDraws an anti-aliased line on a *surface*. If *blend* is True, the\nshades will be be blended with existing pixel shades instead of\noverwriting them.\n\nThe returned pygame2.Rect describes the area affected by\nthe changes."
#define DOC_DRAW_AALINES "aalines (surface, color, (point1, point2, ...) [, blend]) -> pygame2.Rect\n\nDraws multiple contiguous anti-aliased line segments on a surface.\n\nDraws a sequence of anti-aliased lines with a fixed width of 1px\non a *surface*. If *blend* is True, the shades will be be blended\nwith existing pixel shades instead of overwriting them.\n\nThe returned pygame2.Rect describes the area affected by\nthe changes."
#define DOC_DRAW_AAPOLYGON "aapolygon (surface, color, (point1, point2, ...) [, blend]) -> pygame2.Rect\n\nDraws an anti-aliased polygonal shape on a surface.\n\nDraws an polygonal shape on a *surface* with an fixed line width of\n1px. If *blend* is true, the shades will be be blended with existing\npixel shades instead of overwriting them.\n\nThe returned pygame2.Rect describes the area affected by the\nchanges."
#define DOC_DRAW_ARC "arc (surface, color, rect, startangle, stopangle[, width]) -> pygame2.Rect\n\nDraws an elliptical arc on a surface.\n\nDraws an elliptical arc on a *surface* with an optionally\nspecified line width. If the *width* argument is omitted, the line\nwill be 1px thick. If *width* is set to 0, a filled shape will be\ndrawn. The *startangle* and *stopangle* arguments are the initial\nand final angle in degrees.\n\nThe returned pygame2.Rect describes the area affected by the\nchanges."
#define DOC_DRAW_CIRCLE "circle (surface, color, x, y, radius[, width]) -> pygame2.Rect\ncircle (surface, color, point, radius[, width]) -> pygame2.Rect\n\nDraws a circular shape on a surface.\n\nDraws a circular shape on a *surface* with an optionally specified\nline width. If the *width* argument is omitted, the line will be\n1px thick. If *width* is set to 0, a filled circular shape will be\ndrawn. The *x*, *y* point denotes the center of the circle.\n\nThe returned pygame2.Rect describes the area affected by the\nchanges."
#define DOC_DRAW_ELLIPSE "ellipse (surface, color, rect[, width]) -> pygame2.Rect\n\nDraws an elliptical shape on a surface.\n\nDraws an elliptical shape on a *surface* with an optionally\nspecified line width. If the *width* argument is omitted, the line\nwill be 1px thick. If *width* is set to 0, a filled elliptical\nshape will be drawn. The passed pygame2.Rect denotes the\narea, the elliptical shape will be drawn in.\n\nThe returned pygame2.Rect describes the area affected by the\nchanges."
#define DOC_DRAW_LINE "line (surface, color, x1, y1, x2, y2[, width]) -> pygame2.Rect\nline (surface, color, point1, point2[, width]) -> pygame2.Rect\n\nDraws a straight line segment on a surface.\n\nDraws a straight line segment on a *surface* with an optionally\nspecified line width. If the *width* argument is omitted, the line\nwill be 1px thick. There are no endcaps, the ends are squared off\nfor thick lines.\n\nThe returned pygame2.Rect describes the area affected by the\nchanges."
#define DOC_DRAW_LINES "lines (surface, color, (point1, point2, ...) [, width]) -> pygame2.Rect\n\nDraws multiple contiguous line segments on a surface.\n\nDraws a sequence of lines on a *surface* with an optionally\nspecified line width. If the *width* argument is omitted, the\nlines will be 1px thick.\n\nThe returned pygame2.Rect describes the area affected by the\nchanges."
#define DOC_DRAW_POLYGON "polygon (surface, color, (point1, point2), ...) [, width]) -> pygame2.Rect\n\nDraws a polygonal shape on a surface.\n\nDraws a polygonal shape on a *surface* with an optionally\nspecified line width. If the *width* argument is omitted, the line\nwill be 1px thick. If *width* is set to 0, a filled polygon will\nbe drawn.\n\nThe returned pygame2.Rect describes the area affected by the\nchanges."
#define DOC_DRAW_RECT "rect (surface, color, rect [, width]) -> pygame2.Rect\n\nDraws a rectangular shape on a surface.\n\nDraws a rectangular shape on a *surface* with an optionally\nspecified line width. If the *width* argument is omitted, the line\nwill be 1px thick. If the *width* is set to 0, a filled\nrectangular shape will be drawn.\n\nThe returned pygame2.Rect describes the area affected by the\nchanges."

#endif
