#ifndef _PYGAME2_DOCJOYSTICK_H_
#define _PYGAME2_DOCJOYSTICK_H_

#define DOC_JOYSTICK "The joystick module manages the joystick devices on a computer\n(there can be more than one). Joystick devices include trackballs\nand video-game-style gamepads, and the module allows the use of\nmultiple buttons and \"hats\"."
#define DOC_JOYSTICK_GET_NAME "get_name (index) -> str\n\nGets the physical device name of a Joystick.\n\nGets the physical device name of a Joystick. The *index*\nspecifies the device to get the name for and must be in the range [0,\nnum_joysticks - 1]."
#define DOC_JOYSTICK_INIT "init () -> None\n\nInitializes the joystick subsystem of the SDL library."
#define DOC_JOYSTICK_NUM_JOYSTICKS "um_joysticks () -> int\n\nGets the number of detected and available joysticks for the system."
#define DOC_JOYSTICK_OPENED "opened (index) -> bool\n\nGets, whether the specified joystick is opened for access or not.\n\nGets, whether the specified joystick is opened for access or\nnot. The *index* specifies the joystick device to get the state for\nand must be in the range [0, num_joysticks - 1]."
#define DOC_JOYSTICK_QUIT "quit () -> None\n\nShuts down the joystick subsystem of the SDL library.\n\nShuts down the joystick subsystem of the SDL library and closes all\nexisting Joystick objects (leaving them intact).\n\nAfter calling this function, you should not invoke any class,\nmethod or function related to the joystick subsystem as they are\nlikely to fail or might give unpredictable results."
#define DOC_JOYSTICK_EVENT_STATE "event_state (state) -> int\n\nEnables or disable joystick event polling.\n\nEnables or disables joystick event processing. If the joystick\nevent processing is disabled, you will have to update the joystick\nstates manually using update and read the information\nmanually using the specific attributes and methods. Otherwise,\njoystick events are consumed and process by the\n:mod:pygame2.sdl.event module.\n\nThe *state* argument can be ENABLE or IGNORE for\nenabling or disabling the event processing or QUERY to receive\nthe current state. The return value also will be one of those three\nconstants."
#define DOC_JOYSTICK_UPDATE "update () -> None\n\nUpdates the joystick states (in case event processing is disabled)."
#define DOC_JOYSTICK_WAS_INIT "was_init () -> bool\n\nReturns, whether the joystick subsystem of the SDL library is\ninitialized."
#define DOC_JOYSTICK_JOYSTICK "Joystick (index) -> Joystick\n\nCreates a new Joystick object for the specified physical device.\n\nThe Joystick object allows you to access information about the\nunderlying physical device, such as the axes, hats and button\nstates.\n\nThe *index* must be in the range [0,\npygame2.sdl.joystick.num_joysticks - 1].  This will also\nopen the Joystick initially."
#define DOC_JOYSTICK_JOYSTICK_INDEX "Gets the physical device index of the Joystick."
#define DOC_JOYSTICK_JOYSTICK_NAME "Gets the physical device name of the Joystick."
#define DOC_JOYSTICK_JOYSTICK_NUM_AXES "Gets the number of axes available on the Joystick."
#define DOC_JOYSTICK_JOYSTICK_NUM_BALLS "Gets the number of balls available on the Joystick."
#define DOC_JOYSTICK_JOYSTICK_NUM_BUTTONS "Gets the number of buttons available on the Joystick."
#define DOC_JOYSTICK_JOYSTICK_NUM_HATS "Gets the number of hats available on the Joystick."
#define DOC_JOYSTICK_JOYSTICK_OPENED "Gets, whether the Joystick is open or not."
#define DOC_JOYSTICK_JOYSTICK_CLOSE "close () -> None\n\nCloses the access to the underlying joystick device.\n\nCalling or accessing any other method or attribute of the\nJoystick after closing it will cause an exception to be\nthrown. To reaccess the Joystick, you have to open it\nagain."
#define DOC_JOYSTICK_JOYSTICK_GET_AXIS "get_axis (index) -> int\n\nGets the current position of the specified axis.\n\nGets the current position of the specified axis. The axis *index*\nmust be a valid value within the range [0, num_axes - 1]."
#define DOC_JOYSTICK_JOYSTICK_GET_BALL "get_ball (index) -> int, int\n\nGets the relative movement of a trackball.\n\nGets the relative movement of a trackball since the last call to\nget_ball. The ball *index* must be a valid value within the\nrange [0, num_balls - 1]."
#define DOC_JOYSTICK_JOYSTICK_GET_BUTTON "get_button (index) -> bool\n\nGets the state of a button.\n\nGets the current pressed state of a button. The button *index*\nmust be a valid value within the range [0, num_buttons - 1]."
#define DOC_JOYSTICK_JOYSTICK_GET_HAT "get_hat (index) -> int\n\nGets the state of a hat.\n\nGets the current state of a hat. The return value will be a\nbitwise OR'd combination of the hat constants as specified in the\n:mod:pygame2.sdl.constants module. The hat *index* must be a valid\nvalue within the range [0, num_hats - 1]."
#define DOC_JOYSTICK_JOYSTICK_OPEN "open () -> None\n\nOpens the (closed) Joystick.\n\nOpens the (closed) Joystick. If the Joystick is already open, this\nmethod will have no effect."

#endif
