#ifndef _PYGAME2_DOCVIDEO_H_
#define _PYGAME2_DOCVIDEO_H_

#define DOC_VIDEO "SDL video display wrapper module."
#define DOC_VIDEO_GET_DRIVERNAME "get_drivername () -> str\n\nGets the name of the video driver.\n\nGets the name of the video driver or None, if the video system has\nnot been initialised or it could not be determined."
#define DOC_VIDEO_GET_GAMMARAMP "get_gammaramp () -> (int, int, ...), (int, int, ...), (int, int, ...)\n\nGets the color gamma lookup tables for the display.\n\nGets the color gamma lookup table for the display. This will\nreturn three tuples for the red, green and blue gamma values. Each\ntuple contains 256 values."
#define DOC_VIDEO_GET_INFO "get_info () -> dict\n\nGets information about the video hardware.\n\nGets information about the video hardware. The returned dictionary\ncontains the following entries.\n\n+------------------+---------------------------------------------+\n| Entry            | Meaning                                     |\n+==================+=============================================+\n| hw_available     | Is it possible to create hardware surfaces? |\n+------------------+---------------------------------------------+\n| wm_available     | Is a window manager available?              |\n+------------------+---------------------------------------------+\n| blit_hw          | Are hardware to hardware blits accelerated? |\n+------------------+---------------------------------------------+\n| blit_hw_CC       | Are hardware to hardware colorkey blits     |\n|                  | accelerated?                                |\n+------------------+---------------------------------------------+\n| blit_hw_A        | Are hardware to hardware alpha blits        |\n|                  | accelerated?                                |\n+------------------+---------------------------------------------+\n| blit_sw          | Are software to hardware blits accelerated? |\n+------------------+---------------------------------------------+\n| blit_sw_CC       | Are software to hardware colorkey blits     |\n|                  | accelerated?                                |\n+------------------+---------------------------------------------+\n| blit_sw_A        | Are software to hardware alpha blits        |\n|                  | accelerated?                                |\n+------------------+---------------------------------------------+\n| blit_fill        | Are color fills accelerated?                |\n+------------------+---------------------------------------------+\n| video_mem        | Total amount of video memory i" \
"n Kilobytes   |\n+------------------+---------------------------------------------+\n| vfmt             | Pixel format of the video device            |\n+------------------+---------------------------------------------+"
#define DOC_VIDEO_GET_VIDEOSURFACE "get_videosurface () -> Surface\n\nGets the current display surface or None, if there is no such Surface."
#define DOC_VIDEO_INIT "init () -> None\n\nInitializes the video subsystem of the SDL library."
#define DOC_VIDEO_IS_MODE_OK "is_mode_ok (width, height[, bpp, flags]) -> bool\nis_mode_ok (size[, bpp, flags]) -> bool\n\nChecks, whether the requested video mode is supported.\n\nChecks, whether the video mode is supported for the passed *size*,\nbit depth and *flags*. If the bit depth (*bpp*) argument is omitted,\nthe current screen bit depth will be used.\n\nThe optional *flags* argument is the same as for set_mode."
#define DOC_VIDEO_LIST_MODES "list_modes ([, format, flags]) -> [rect, rect, ...]\n\nReturns the supported modes for a specific format and flags.\n\nReturns the supported modes for a specific *format* and *flags*.\nThe optional *format* argument must be a PixelFormat\ninstance with the desired mode information. The optional *flags*\nargument is the same as for set_mode.\n\nIf both, the *format* and *flags* are omitted, all supported screen\nresolutions for all supported formats and flags are returned."
#define DOC_VIDEO_QUIT "quit () -> None\n\nShuts down the video subsystem of the SDL library.\n\nAfter calling this function, you should not invoke any class,\nmethod or function related to the video subsystem as they are\nlikely to fail or might give unpredictable results."
#define DOC_VIDEO_SET_GAMMA "set_gamma (red, green, blue) -> None\n\nSets the gamma values for all three color channels.\n\nSets the gamma values for all three color channels. In case\nadjusting the gamma is not supported, an exception will be raised."
#define DOC_VIDEO_SET_GAMMARAMP "set_gammaramp (redtable, greentable, bluetable) -> None\n\nSets the color gamma lookup tables for the display.\n\nSets the color gamma lookup table for the display. The three\narguments must be sequences with 256 integer value enties for the\ngamma ramps."
#define DOC_VIDEO_SET_MODE "set_mode (width, height[, bpp, flags]) -> Surface\nset_mode (size[, bpp, flags]) -> Surface\n\nCreates the main display Surface.\n\nCreates the main display Surface using the specified *size*, bit\ndepth and *flags*. If the bit depth (*bpp*) argument is omitted, the\ncurrent screen bit depth will be used.\n\nThe optional *flags* argument can be a bitwise OR'd combination of\nthe surface flags from the :mod:pygame2.sdl.constants."
#define DOC_VIDEO_WAS_INIT "was_init () -> bool\n\nReturns, whether the video subsystem of the SDL library is initialized."
#define DOC_VIDEO_OVERLAY "Overlay (surface, width, height, format) -> Overlay\nOverlay (surface, size, format) -> Overlay\n\nCreates a new Overlay for a surface.\n\nThe Overlay objects provide support for accessing hardware video\noverlays. Video overlays do not use standard RGB pixel formats, and can\nuse multiple resolutions of data to create a single image.\n\nThe Overlay objects represent lower level access to the display\nhardware. To use the object you should understand the technical details\nof video overlays.\n\nThe Overlay *format* determines the type of pixel data used. Not all\nhardware will support all types of overlay formats. Supported\n*format* types are listed in the :mod:pygame2.sdl.constants\nmodule.\n\nThe *width* and *height* arguments control the size for the overlay image\ndata. The overlay image can be displayed at any size, not just the\nresolution of the overlay and are always visible, and always show above\nthe regular display contents."
#define DOC_VIDEO_OVERLAY_FORMAT "Gets the format of the Overlay."
#define DOC_VIDEO_OVERLAY_H "Gets the height of the Overlay."
#define DOC_VIDEO_OVERLAY_HEIGHT "Gets the height of the Overlay."
#define DOC_VIDEO_OVERLAY_HW_OVERLAY "Gets, whether the Overlay is hardware accelerated."
#define DOC_VIDEO_OVERLAY_LOCKED "Gets, whether the Overlay is currently locked."
#define DOC_VIDEO_OVERLAY_PITCHES "Gets the pitches (length in bytes) for all planes of the\nOverlay."
#define DOC_VIDEO_OVERLAY_PIXELS "Gets the pixel buffers for all planes of the Overlay."
#define DOC_VIDEO_OVERLAY_PLANES "Get the amount of planes for the Overlay."
#define DOC_VIDEO_OVERLAY_SIZE "Gets the width and height of the Overlay."
#define DOC_VIDEO_OVERLAY_W "Gets the width of the Overlay."
#define DOC_VIDEO_OVERLAY_WIDTH "Gets the width of the Overlay."
#define DOC_VIDEO_OVERLAY_DISPLAY "display ([rect]) -> None\n\nDisplays the current overlay data in the SDL overlay plane."
#define DOC_VIDEO_OVERLAY_LOCK "lock () -> None\n\nLocks the Overlay for a direct access to its internal pixel data."
#define DOC_VIDEO_OVERLAY_UNLOCK "unlock () -> None\n\nUnlocks the Overlay, releasing the direct access to the pixel data."
#define DOC_VIDEO_PIXELFORMAT "PixelFormat () -> PixelFormat\n\nCreates a new, empty PixelFormat.\n\nThe PixelFormat typically describes a Surface's pixel buffer\nrepresentation. It contains information about the bit depth, RGBA masks,\nthe palette and transparency information."
#define DOC_VIDEO_PIXELFORMAT_ALPHA "Gets or sets the overall surface alpha value in the\nrange [0, 255]."
#define DOC_VIDEO_PIXELFORMAT_BITS_PER_PIXEL "Gets or sets the bits per pixel to use for\nstoring a RGBA value."
#define DOC_VIDEO_PIXELFORMAT_BYTES_PER_PIXEL "Gets or sets the bytes per pixel to use for\nstoring a RGBA value."
#define DOC_VIDEO_PIXELFORMAT_COLORKEY "Gets or sets the RGBA value of transparent pixels."
#define DOC_VIDEO_PIXELFORMAT_LOSSES "Gets or sets the precision loss of each RGBA color\ncomponent."
#define DOC_VIDEO_PIXELFORMAT_MASKS "Gets or sets the binary masks used to retrieve individual\ncolor values."
#define DOC_VIDEO_PIXELFORMAT_PALETTE "Gets the color palette (if any) used by the PixelFormat. If the\nPixelFormat does not have any palette, None will be returned."
#define DOC_VIDEO_PIXELFORMAT_READONLY "Gets, whether the PixelFormat is read-only (this cannot be\nchanged)."
#define DOC_VIDEO_PIXELFORMAT_SHIFTS "Gets the binary left shift of each color component in\nthe pixel value."
#define DOC_VIDEO_PIXELFORMAT_GET_RGBA "get_rgba (color) -> pygame2.Color\n\nGets the best matching pygame2.Color for the passed color.\n\nGets a color value, which fits the PixelFormat best. This means\nthat an internal conversion is done (on demand) to match the\npassed color to the PixelFormat's supported value ranges. If\nthe PixelFormat does not have alpha transparency support, the\ncolor's alpha value will be set to fully opaque (255)."
#define DOC_VIDEO_PIXELFORMAT_MAP_RGBA "map_rgba (r, g, b[, a]) -> long\nmap_rgba (color) -> long\n\nConverts a color to the best value matching the format.\n\nGets a color value, which fits the PixelFormat best. This means\nthat an internal conversion is done (on demand) to match the\npassed color to the PixelFormat's supported value ranges.\nInstead of returning a color as in get_rgba, an integer value\nmatching the PixelFormat's supported value ranges will be\nreturned."
#define DOC_VIDEO_SURFACE "Surface (width, height[, depth, flags, masks]) -> Surface\nSurface (size[, depth, flags, masks]) -> Surface\n\nCreates a new Surface to represent image data.\n\nCreates a new Surface which represents a rectangular area of pixel\ninformation, typically displaying some sort of image data. The Surface is\nthe central element of the :mod:pygame2.sdl.video module and used to\ndisplay image information on an SDL screen window.\n\n.. todo\n\nTODO"
#define DOC_VIDEO_SURFACE_CLIP_RECT "Gets or sets the current clipping rectangle for\noperations on the Surface."
#define DOC_VIDEO_SURFACE_FLAGS "The currently set flags for the Surface."
#define DOC_VIDEO_SURFACE_FORMAT "Gets the (read-only) PixelFormat for this Surface."
#define DOC_VIDEO_SURFACE_H "Gets the height of the Surface."
#define DOC_VIDEO_SURFACE_HEIGHT "Gets the height of the Surface."
#define DOC_VIDEO_SURFACE_LOCKED "Gets, whether the Surface is currently locked."
#define DOC_VIDEO_SURFACE_PITCH "Get the length of a surface scanline in bytes."
#define DOC_VIDEO_SURFACE_PIXELS "Gets the pixel buffer of the Surface."
#define DOC_VIDEO_SURFACE_SIZE "Gets the size of the Surface."
#define DOC_VIDEO_SURFACE_W "Gets the width of the Surface."
#define DOC_VIDEO_SURFACE_WIDTH "Gets the width of the Surface."
#define DOC_VIDEO_SURFACE_BLIT "blit (srcsurface[, destrect, srcrect, blendargs]) -> Rect\n\nDraws the passed source surface onto this surface.\n\nDraws the passed source surface onto this surface. The *destrect* and\n*srcrect* arguments are used for clipping the destination area (on this\nsurface) and source area (on the source surface). For the destination\nrectangle, the width and height are ignored, only the position is taken\ninto account. For the source rectangle, all values, position as well as\nthe size are used for clipping.\n\nThe optional *blendargs* for the drawing operation perform  certain\nspecialised pixel manipulations. For those, pixels on the same\nposition are evaluated and the result manipulated according to the\nargument.\n\nFor all supported blending operations take a look at the blending\nconstants in the :mod:pygame2.sdl.constants module."
#define DOC_VIDEO_SURFACE_COPY "copy () -> Surface\n\nCreates an exact copy of the Surface and its image data."
#define DOC_VIDEO_SURFACE_CONVERT "convert ([pixelformat, flags]) -> Surface\n\nConverts the Surface to the desired pixel format.\n\nConverts the Surface to the desired pixel format. If no format\nis given, the active display format is used, which will be the fastest\nfor blit operations to the screen. The flags are the same as for surface\ncreation.\n\nThis creates a new, converted surface and leaves the original one\nuntouched."
#define DOC_VIDEO_SURFACE_FILL "fill (color[, dstrect, blendargs]) -> None\n\nFills the Surface with a color.\n\nFills the Surface with the desired color. The color does not\nneed to  match the Surface's format, it will be converted\nimplicitly to the nearest appropriate color for its format.\n\nThe optional destination rectangle limits the color fill to the\nspecified area. The blendargs are the same as for the blit\noperation, but compare the color with the specific Surface pixel value."
#define DOC_VIDEO_SURFACE_FLIP "flip () -> None\n\nSwaps the screen buffers for the Surface.\n\nSwaps screen buffers for the Surface, causing a full update and redraw\nof its whole area."
#define DOC_VIDEO_SURFACE_GET_ALPHA "get_alpha () -> int\n\nGets the current overall alpha value of the Surface.\n\nGets the current overall alpha value of the Surface. In case the\nsurface does not support alpha transparency (SRCALPHA flag not set),\nNone will be returned."
#define DOC_VIDEO_SURFACE_GET_AT "get_at (x, y) -> Color\nget_at (point) -> Color\n\nGets the Surface pixel value at the specified point."
#define DOC_VIDEO_SURFACE_GET_COLORKEY "get_colorkey () -> Color\n\nGets the colorkey for the Surface.\n\nGets the colorkey for the Surface or None in case it has no colorkey\n(SRCCOLORKEY flag not set)."
#define DOC_VIDEO_SURFACE_GET_PALETTE "get_palette () -> (Color, Color, ...)\n\nGets the palette colors used by the Surface.\n\nGets the palette colors used by the Surface or None, if the Surface\ndoes not use any palette."
#define DOC_VIDEO_SURFACE_LOCK "lock () -> None\n\nLocks the Surface for a direct access to its internal pixel data."
#define DOC_VIDEO_SURFACE_SAVE "save (file[, type]) -> None\n\nSaves the Surface to a file.\n\nSaves the Surface to a file. The file argument can be either a\nfile name or a file-like object to save the Surface to. The\noptional type argument is required, if the file type cannot be\ndetermined by the suffix.\n\nCurrently supported file types (suitable to pass as string for\nthe type argument) are:\n\n* BMP\n* TGA\n* PNG\n* JPEG (JPG)\n\nIf no type information is supplied and the file type cannot be\ndetermined either, it will use TGA.\n\n.. note\n\nThe file object *must* support binary read and write\naccess. This is especially important for Python 3.x users."
#define DOC_VIDEO_SURFACE_SCROLL "scroll ([dx, dy]) -> None\n\nScrolls the Surface in place.\n\nMove the Surface contents by *dx* pixels right and *dy* pixels\ndown. dx and dy may be negative for left and up scrolls\nrespectively. Areas of the surface that are not overwritten\nretain their original pixel values. Scrolling is contained by\nthe Surface clip area. It is safe to have *dx* and *dy* values\nthat exceed the surface size."
#define DOC_VIDEO_SURFACE_SET_ALPHA "set_alpha (alpha[, flags]) -> None\n\nAdjusts the alpha properties of the Surface.\n\nTODO"
#define DOC_VIDEO_SURFACE_SET_AT "set_at (x, y, color) -> None\nset_at (point, color) -> None\n\nSets the Surface pixel value at the specified point."
#define DOC_VIDEO_SURFACE_SET_COLORKEY "set_colorkey (color[, flags]) -> None\n\nAdjusts the colorkey of the Surface.\n\nSets the colorkey of the Surface to the passed value. if *flags* is\nomitted, SRCCOLORKEY will be used, if *flags* is set to 0, the colorkey\nwill be reset.\n\nThe colorkey denotes a certain pygame2.Color, which will be\nused for emulating transparency. Each pixel on the Surface matching the\nset color will be ignored on blit operations."
#define DOC_VIDEO_SURFACE_SET_COLORS "set_colors ((color1, color2, ...)[, first]) -> bool\n\nSets a portion of the colormap palette for the 8-bit Surface.\n\nSets a portion of the colormap palette for the 8-bit Surface,\nstarting at the desired first position. If the first position\nplus length of the passed colormap exceeds the Surface palette\nsize, the palette will be unchanged and False returned.\n\nIf any other error occurs, False will be returned and the\nSurface palette should be inspected for any changes."
#define DOC_VIDEO_SURFACE_SET_PALETTE "set_palette ((color1, color2, ...)[, flags, first]) -> bool\n\nSets a portion of the palette for the given 8-bit surface.\n\nSets a portion of the color palette for the 8-bit Surface,\nstarting at the desired *first* position. If the *first* position\nplus length of the passed colormap exceeds the Surface palette\nsize, the palette will be unchanged and False returned.\n\nIf *flags* is omitted and a Surface with hardware palette used, both\nthe physical and logical palettes will be modified. Otherwise, *flags*\ncan be set to LOGPAL or PHYSPAL or a combination to modify either the\nlogical, physical or both palettes.\n\nIf any other  error occurs, False will be returned and the Surface\npalette should be inspected for any changes."
#define DOC_VIDEO_SURFACE_UNLOCK "unlock () -> None\n\nUnlocks the Surface, releasing the direct access to the pixel data."
#define DOC_VIDEO_SURFACE_UPDATE "update ([rect]) -> None\nupdate ([(rect1, rect2, ...)]) -> None\n\nUpdates the given area on the Surface.\n\nUpates the given area (or areas, if a list of rects is passed)\non the Surface, performing partial screen buffer swaps.\nFor smaller portions of the Surface, this is much faster than running\nflip."

#endif
