#ifndef _PYGAME2_DOCBASE_H_
#define _PYGAME2_DOCBASE_H_

#define DOC_BASE "This module allows for rendering all font formats supported by\nFreeType, namely TTF, Type1, CFF, OpenType, SFNT, PCF, FNT, BDF,\nPFR and Type42 fonts.\n\nThis module is optional, and replaces all of the functionality of the\noriginal 'font' module, whilst expanding it. This module depends in no\nway on the SDL_ttf library.\n\nYou should test that pygame.freetype is initialized before attempting\nto use the module; if the module is available and loaded, it will be\nautomatically initialized by pygame.init()\n\nMost of the work done with fonts is done by using the actual Font objects.\nThe module by itself only has routines to initialize itself and create\nFont objects with pygame.freetype.Font().\n\nYou can load fonts from the system by using the pygame.freetype.SysFont()\nfunction. There are a few other functions to help lookup the system fonts.\n\nPygame comes with a builtin default font. This can always be accessed by\npassing None as the font name to the Font constructor."
#define DOC_BASE_GET_ERROR "get_error() -> string\n\nReturns the description of the last error which occurred in the\nFreeType library, or None if no errors have occurred."
#define DOC_BASE_GET_VERSION "get_version() -> (int, int, int)\n\nGets the version of the FreeType2 library which was used to build\nthe 'freetype' module.\n\nNote that the FreeType module depends on the FreeType 2 library,\nand will not compile with the original FreeType 1.0, hence the first\nelement of the tuple will always be 2."
#define DOC_BASE_INIT "init(default_cache_size=64) -> None\n\nInitializes the underlying FreeType 2 library.\n\nThis function must be called before trying to use any of the functionality\nof the 'freetype' module. It is safe to call this function more than once.\n\nOptionally, you may specify a default size for the Glyph cache:\nthis is the maximum amount of glyphs that will be cached at any\ngiven time by the module.\nExceedingly small values will be automatically tuned for performance."
#define DOC_BASE_QUIT "quit() -> None\n\nShuts down the underlying FreeType 2 library.\n\nAfter calling this function, you should not invoke any class,\nmethod or function related to the 'freetype' module as they are likely to\nfail or might give unpredictable results. It is safe to call this function\neven if the module hasn't been initialized yet."
#define DOC_BASE_WAS_INIT "was_init() -> bool\n\nReturns whether the the FreeType 2 library is initialized."
#define DOC_BASE_FONT "Font (file [, ptsize, style, index]) -> Font\n\nCreates a new Font from a supported font file.\n\n*file* can be either a string representing the font's filename,\na file-like object containing the font, or None; in this last case\nthe default, built-in font will be used.\n\nOptionally, a *ptsize* argument may be specified to set the\ndefault size in points which will be used to render the font.\nSuch size can also be specified manually on each method call.\nBecause of the way the caching system works, specifying a\ndefault size on the constructor doesn't imply a performance\ngain over manually passing the size on each function call.\n\nIf the font file has more than one face, the *index* argument\nmay be specified to specify which face index to load. Defaults\nto 0; font loading will fail if the given index is not contained\nin the font.\n\nThe 'style' argument will set the default style (italic,\nunderline, bold) used to draw this font. This style may be\noverriden on any Font.render() call."
#define DOC_BASE_FONT_NAME "Read only. Gets the name of the font face."
#define DOC_BASE_FONT_HEIGHT "Read only. Gets the height of the Font. This is the average\nvalue of all glyphs in the font."
#define DOC_BASE_FONT_STYLE "Gets or sets the default style of the Font. This default style\nwill be used for all text rendering and size calculations unless\noverriden specifically in the render() or get_size() calls.\nThe style value may be a bitwise OR of one or more of the following\nconstants:\n\nSTYLE_NONE\nSTYLE_UNDERLINE\nSTYLE_ITALIC\nSTYLE_BOLD\n\nThese constants may be found on the FreeType constants module.\nOptionally, the default style can be modified or obtained\naccessing the individual style attributes (underline, italic,\nbold)."
#define DOC_BASE_FONT_UNDERLINE "Gets or sets whether the font will be underlined when drawing text.\nThis default style value will be used for all text rendering and\nsize calculations unless overriden specifically in the render()\nor get_size() calls, via the 'style' parameter."
#define DOC_BASE_FONT_BOLD "Gets or sets whether the font will be bold when drawing text.\nThis default style value will be used for all text rendering and\nsize calculations unless overriden specifically in the render()\nor get_size() calls, via the 'style' parameter."
#define DOC_BASE_FONT_ITALIC "Gets or sets whether the font will be slanted when drawing text.\nThis default style value will be used for all text rendering and\nsize calculations unless overriden specifically in the render()\nor get_size() calls, via the 'style' parameter."
#define DOC_BASE_FONT_FIXED_WIDTH "Read only. Returns whether this Font is a fixed-width (bitmap) font.\n\n.. note\n\nNote that scalable fonts whose glyphs are all the same width\n(i.e. monospace TTF fonts used for programming) are not considered\nfixed width."
#define DOC_BASE_FONT_ANTIALIASED "Gets or sets the font's antialiasing mode. This defaults to True\non all fonts, which will be rendered by default antialiased.\n\nSetting this to true will change all rendering methods to use\nglyph bitmaps without antialiasing, which supposes a small speed gain\nand a significant memory gain because of the way glyphs are cached."
#define DOC_BASE_FONT_VERTICAL "Gets or sets whether the font is a vertical font such as fonts\nrepresenting Kanji glyphs or other styles of vertical writing.\n\nChanging this attribute will cause the font to be rendering\nvertically, and affects all other methods which manage glyphs\nor text layouts to use vertical metrics accordingly.\n\nNote that the FreeType library doesn't automatically detect\nwhether a font contains glyphs which are always supposed to\nbe drawn vertically, so this attribute must be set manually\nby the user.\n\nAlso note that several font formats (specially bitmap based\nones) don't contain the necessary metrics to draw glyphs\nvertically, so drawing in those cases will give unspecified\nresults."
#define DOC_BASE_FONT_GET_SIZE "get_size(text [, style, rotation, ptsize]) -> int, int\n\nGets the size in pixels which 'text' will occupy when rendered\nusing this Font. The calculations will take into account the\nfont's default style (e.g. underlined fonts take extra height\nfor the underline), or the style may be overriden by the\n'style' parameter.\n\nReturns a tuple containing the width and height of the text's\nbounding box.\n\nThe calculations are done using the font's default size in points,\nwithout any rotation, and taking into account fonts which are set\nto be drawn vertically via the Font.vertical attribute.\nOptionally you may specify another point size to use via the\n'ptsize' argument, or a text rotation via the 'rotation' argument."
#define DOC_BASE_FONT_GET_METRICS "get_metrics(text, [bbmode, ptsize]) -> [(...), ...]\n\nReturns the glyph metrics for each character in 'text'.\n\nThe glyph metrics are returned inside a list; each character will\nbe represented as a tuple inside the list with the following values:\n\n(min_x, max_x, min_y, max_y, horizontal_advance)\n\nBy default, these values are returned as grid-fitted pixel\ncoordinates (ints). Optionally, one of the following constants\n(which can be found on the FreeType constants module) may be passed\nas the bbmode argument:\n\nBBOX_EXACT: Return accurate floating point values.\n\nBBOX_EXACT_GRIDFIT: Return accurate floating point values aligned\nto the drawing grid.\n\nBBOX_PIXEL: Return pixel coordinates (ints).\n\nBBOX_PIXEL_GRIDFIT (default): Return grid-aligned pixel coordinates.\n\nThe calculations are done using the font's default size in points.\nOptionally you may specify another point size to use."
#define DOC_BASE_FONT_RENDER "render(dest, text, fgcolor [, bgcolor, style, rotation, ptsize]) -> int, int[, pygame2.sdl.video.Surface]\n\nRenders a text on a SDL surface.\n\nRenders the string *text* to a\npygame2.sdl.video.Surface, using the color *fgcolor*.\n\nThe *dest* parameter is supposed to be a tuple containing\nthe surface and the coordinates at which the text will be rendered,\nin that order.\n\nIf such tuple exists, and the destination surface is a valid\npygame2.sdl.video.Surface (independently of its bit\ndepth), the text will be rendered directly on top of it at the\npassed coordinates, using the given *fgcolor*, and painting\nthe background of the text with the given 'bgcolor', if available.\nThe alpha values for both colors are always taken into account.\nThe width and height of the rendered text will be returned in a tuple.\n\nIf 'None' is passed instead of the destination tuple, a new 32 bit\npygame2.sdl.video.Surface will be created with the\nrequired size to contain the drawn text, and using *bgcolor* as its\nbackground color. If a background color is not available, the\nsurface will be filled with zero alpha opacity.\nThe width and height of the rendered text,\ntogether with the new pygame2.sdl.video.Surface, will\nbe returned in a tuple.\n\nThe rendering is done using the font's default size in points\nand its default style, without any rotation, and taking into\naccount fonts which are set to be drawn vertically via the\nFont.vertical attribute.\n\nOptionally you may specify another point size to use via the\n'ptsize' argument, a text rotation via the 'rotation' argument,\nor a new text style via the 'style' argument.\n\nThis function is only available when PyGame 2 has been compiled\nwith SDL support."
#define DOC_BASE_FONT_RENDER_RAW "render_raw(text [, ptsize]) -> int, int, bytes\n\nRenders a text to a byte buffer.\n\nRenders the string *text* to a raw buffer of bytes, each byte\nrepresenting the opacity of the glyph's raster image in\ngrayscale.\n\nThe width (pitch) and height of the rendered text, together with\nthe bytes buffer, will be returned in a tuple.\n\nThe rendering is done using the font's default size in points.\nOptionally you may specify another point size to use."

#endif
