#ifndef _PYGAME2_DOCEVENT_H_
#define _PYGAME2_DOCEVENT_H_

#define DOC_EVENT "Pygame SDL event wrapper module. The event module manages the SDL\nrelated event system and allows one to receive, send and manipulate\nevents.\n\nThe event module is the main messaging system for SDL windows and\nuses a queue to deal with events such as mouse and keyboard input,\nwindow state changes or user-defined ones."
#define DOC_EVENT_CLEAR "clear ([events]) -> None\n\nClears the event queue from certain event types.\n\nClears the event queue from certain event types. If no argument is\npassed, all current events are removed from the queue. Otherwise\nthe argument can be a sequence or a bitmask combination of event\ntypes to clear."
#define DOC_EVENT_GET "get ([events]) -> [Event, Event, ... ]\n\nGets events from the event queue.\n\nGets the current events from the event queue. If no argument is\npassed, all currently available events are received from the event\nqueue and returned as list. Otherwise, the argument can be a\nsequence or a bitmask combination of event types to receive from\nthe queue.\n\nIf no matching events are found on the queue, an empty list will be\nreturned."
#define DOC_EVENT_GET_APP_STATE "get_app_state () -> int\n\nGets the current application state.\n\nGets the current application state. This will be a bitmask\ncombination of the APPMOUSEFOCUS, APPINPUTFOCUS or\nAPPACTIVE masks, indicating whether the application currently is\nactive and has the mouse and keyboard input focus."
#define DOC_EVENT_GET_BLOCKED "get_blocked () -> [int, int, ...]\n\nGets a list of currently blocked event types.\n\nGets a list of currently blocked event types. Events having the\nmatching type will not be processed by the event queue."
#define DOC_EVENT_GET_FILTER "get_filter () -> object\n\nGets the currently set filter hook method.\n\nGets the filter hook method set previously by :func:set_filter."
#define DOC_EVENT_PEEK "peek ([events]) -> bool\n\nChecks, whether certain event types are currently on the queue.\n\nChecks, whether certain event types are currently on the queue. If\nno argument is passed, this method simply checks, if there is any\nevent on the event queue. Otherwise, the argument can be a\nsequence or a bitmask combination of event types to check for. In\ncase one event is found, which corresponds to the requested\ntype(s), True is returned."
#define DOC_EVENT_PEEP "peep (num, action, mask[, events]) -> int or [Event, ...]\n\nChecks the event queue for events and optionally returns them.\n\nThis is an advanced event queue querying and manipulation\nmethod. It allows to inspect the event queue, to receive events\nfrom it or to add events.\n\nTODO"
#define DOC_EVENT_POLL "poll () -> Event\n\nGets a single event from the event queue.\n\nReturns a single event from the queue. If the event queue is\nempty, None will be returned. If an event is available and\nreturned, it will be removed from the queue."
#define DOC_EVENT_PUMP "pump () -> None\n\nPumps the event queue, forcing it to gather pending events from\ndevices.\n\nThis gathers all pending events and input information from\ndevices and places them on the event queue.\n\nIt only has to be called, if you use :func:peep or a filter hook\nwithout using another event function or no other event function at\nall."
#define DOC_EVENT_PUSH "push (event) -> None\n\nPlaces a new event at the end of the event queue.\n\nThis is usually used for placing user defined events on the event\nqueue. You also can push user created device events on the queue,\nbut this will not change the state of the device itself."
#define DOC_EVENT_SET_BLOCKED "set_blocked (type) -> None\n\nBlocks a single or multiple event types.\n\nThis will block a single or multiple event types from being\nprocessed by the event system and thus will exactly behave like\n:func:state(type, IGNORE).\n\nIn case other event types are already blocked, the block for them\nwill be reset.\n\nTo remove the block for all events, call set_blocked(None)."
#define DOC_EVENT_SET_FILTER "set_filter (filterhook) -> None\n\nSets up a filter hook for events.\n\nThis sets up a filter to process all events before they are posted\nto the event queue. In order to process events correctly, the\n*filterhook* must return True or False, indicating whether the event\nprocessd by it, is allowed to be placed on the event queue or not.\nIt has to take a single argument, which will be the event to process. \n\ndef example_filter_hook (event):\nif event.type == ... and ...:\n# The event matches a certain scheme, do not allow it.\nreturn False\n# Any other event may pass.\nreturn True\n\nIn case the QUITEVENT is processed by the event filter, returning\nTrue will cause the SDL window to be closed, otherwise, the window\nwill remain open, if possible.\n\n.. note\n\nEvents pushed onto the event queue using :func:push or :func:peep do\nnot get passed through the filter."
#define DOC_EVENT_STATE "state (type, state) -> int\n\nSets the processing state of an event type.\n\nThis allows you to set the processing state of an event *type*.\nIf the *state* is set to IGNORE, events matching *type* will be\nautomatically dropped from the event queue and not be filtered. This will\nbehave similar to :func:set_blocked. If *state* is set to\nENABLE, events matching *type* will be processed normally.\nIf *state* is set to QUERY, this will return the current\nprocessing state of the specified event type (ENABLE or\nIGNORE)."
#define DOC_EVENT_WAIT "wait () -> Event\n\nWaits indefinitely for the next available event.\n\nThis is a blocking method, that only returns, if an event occurs\non the event queue. Once an event occurs, it will be returned to\nthe caller and removed from the queue. While the program is\nwaiting it will sleep in an idle state."
#define DOC_EVENT_EVENT "Event (type, dict) -> Event\n\nCreates a new event to be used in the event messaging system.\n\nThe type can be one of the valid SDL event type constants. Each\nevent carries a certain set of attributes, which are only\navailable for that event within the event object.\n\n+--------------------------+-------------------------------------------+\n| Event type               | Attributes                                |\n+==========================+===========================================+\n|         ACTIVEEVENT      | |  state : The activation state. This will|\n|                          |   be APPMOUSEFOCUS for the mouse focus,   |\n|                          |   APPINPUTFOCUS for keyboard input focus  |\n|                          |   and APPACTIVE on iconifying ( gain  = 0)|\n|                          |   or restoring ( gain  = 1) the window.   |\n|                          | |  gain : 1 on gain, 0 on loss            |\n+--------------------------+-------------------------------------------+\n|         KEYDOWN          | |  state : will always be PRESSED         |\n|                          | |  scancode : The hardware scancode       |\n|                          | |  sym : same as  key                     |\n|                          | |  key : The sdl constant of the key (e.g.|\n|                          |   K_a, K_TAB, ...)                        |\n|                          | |  mod :  The currently pressed modifier  |\n|                          |   keys (e.g. KMOD_SHIFT)                  |\n|                          | |  unicode : The unicode represenation of |\n|                          |   the key.                                |\n+--------------------------+-------------------------------------------+\n|         KEYUP            | |  state : will always be RELEASED        |\n|                          | |  scancode : The hardware scancode       |\n|                          | |" \
"  sym : same as  key                     |\n|                          | |  key : The sdl constant of the key (e.g.|\n|                          |   K_a, K_TAB, ...)                        |\n|                          | |  mod :  The currently pressed modifier  |\n|                          |   keys (e.g. KMOD_SHIFT)                  |\n|                          | |  unicode : The unicode represenation of |\n|                          |   the key.                                |\n+--------------------------+-------------------------------------------+\n|         MOUSEMOTION      | |  state : Bit-wise combination of the    |\n|                          |   button states (see  buttons )           |\n|                          | |  x : The current x-coordinate on the    |\n|                          |   window.                                 |\n|                          | |  y : The current y-coordinate on the    |\n|                          |   window.                                 |\n|                          | |  xrel : relative motion in x direction  |\n|                          |   since the last event poll               |\n|                          | |  yrel : relative motion in y direction  |\n|                          |   since the last event poll               |\n|                          | |  pos : The x/y coordinate as tuple      |\n|                          | |  rel : the relative x/y motion as tuple |\n|                          | |  buttons : 3-value tuple containing the |\n|                          |   mouse button states for the first three |\n|                          |   mouse buttons. Each of the values is    |\n|                          |   either *True* or *False*, indicating    |\n|                          |   whether the button is pressed or not    |\n+--------------------------+-------------------------------------------+\n|     " \
"    MOUSEBUTTONDOWN  | |  state : will always be PRESSED         |\n|                          | |  x : The current x-coordinate on the    |\n|                          |   window.                                 |\n|                          | |  y : The current y-coordinate on the    |\n|                          |   window.                                 |\n|                          | |  pos : The x/y coordinate as tuple      |\n|                          | |  button : The pressed button (will be   |\n|                          |   either BUTTON_LEFT, BUTTON_MIDDLE or    |\n|                          |   BUTTON_RIGHT                            |\n+--------------------------+-------------------------------------------+\n|         MOUSEBUTTONUP    | |  state : will always be RELEASED        |\n|                          | |  x : The current x-coordinate on the    |\n|                          |   window.                                 |\n|                          | |  y : The current y-coordinate on the    |\n|                          |   window.                                 |\n|                          | |  pos : The x/y coordinate as tuple      |\n|                          | |  button : The pressed button (will be   |\n|                          |   either BUTTON_LEFT, BUTTON_MIDDLE or    |\n|                          |   BUTTON_RIGHT                            |\n+--------------------------+-------------------------------------------+\n|         JOYAXISMOTION    | |  which : The device index of the        |\n|                          |   joystick                                |\n|                          | |  joy : same as  which                   |\n|                          | |  axis : The joystick axis index         |\n|                          | |  value : The current position of the    |\n|                          |   axis (between -32768 and " \
"32767)         |\n+--------------------------+-------------------------------------------+\n|         JOYBALLMOTION    | |  which : The device index of the        |\n|                          |   joystick                                |\n|                          | |  joy : same as  which                   |\n|                          | |  ball : The trackball index             |\n|                          | |  xrel : The relative motion in x        |\n|                          |   direction since the last event poll     |\n|                          | |  yrel : The relative motion in y        |\n|                          |   direction since the last event poll     |\n|                          | |  rel : The relative x/y motion as tuple |\n+--------------------------+-------------------------------------------+\n|         JOYHATMOTION     | |  which : The device index of the        |\n|                          |   joystick                                |\n|                          | |  joy : same as  which                   |\n|                          | |  hat : The hat index                    |\n|                          | |  value : The current hat position as    |\n|                          |   bit-wise combination (e.g. HAT_LEFTUP)  |\n+--------------------------+-------------------------------------------+\n|         JOYBUTTONDOWN    | |  which : The device index of the        |\n|                          |   joystick                                |\n|                          | |  joy : same as  which                   |\n|                          | |  button : The button index              |\n|                          | |  state : wil always be PRESSED          |\n+--------------------------+-------------------------------------------+\n|         JOYBUTTONUP      | |  which : The device index of the        |\n|                          |   j" \
"oystick                                |\n|                          | |  joy : same as  which                   |\n|                          | |  button : The button index              |\n|                          | |  state : Will always be RELEASED        |\n+--------------------------+-------------------------------------------+\n|         VIDEORESIZE      | |  w : The width to resize to             |\n|                          | |  h : The height to resize to            |\n|                          | |  size : The width and height as tuple   |\n+--------------------------+-------------------------------------------+\n|         VIDEOEXPOSE      | no attributes                             |\n+--------------------------+-------------------------------------------+\n|         QUIT             | no attributes                             |\n+--------------------------+-------------------------------------------+\n\nSome events have special or system-dependent attributes.\n\n+---------------------+-------------------------------------------------+\n| Event type          | Attributes                                      |\n+=====================+=================================================+\n|         SYSWMEVENT  | |  hwnd ,  msg ,  wparam ,  lparam  for Windows |\n|                     | |  event                            for X11     |\n|                     | | no attributes                     for others  |\n+---------------------+-------------------------------------------------+\n\nFor user-defined events, there is the special USEREVENT type (and\nabove). User-defined events can carry their own type, which must\nbe in the range [USEREVENT, (NUMEVENTS - 1)]. If they\nare sent from pygame code directly, they do not carry specific attributes,\njust those given to them by the passed dictionary.\n\nIf they come from another system (e.g. 3rd party library), they" \
"\nmight carry the following attributes:\n\n+--------------------+--------------------------+\n| Event type         | Attributes               |\n+====================+==========================+\n|         USEREVENT  |  code ,  data1 ,  data2  |\n+--------------------+--------------------------+"
#define DOC_EVENT_EVENT_NAME "Gets the name of the Event."
#define DOC_EVENT_EVENT_TYPE "Gets the type id of the Event."

#endif
